<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @author Valery Fremaux
 * @package admin/tool
 * @subpackage sync
 */

require_once('../../../../config.php');
require_once($CFG->dirroot.'/course/lib.php');
require_once($CFG->libdir.'/adminlib.php');
require_once($CFG->dirroot.'/admin/tool/sync/courses/lib.php');
require_once($CFG->dirroot.'/admin/tool/sync/lib.php');

require_login();

if (!is_siteadmin()) {
    print_error('erroradminrequired', 'tool_sync');
}
if (! $site = get_site()) {
    print_error('errornosite', 'tool_sync');
}
if (!$adminuser = get_admin()) {
    print_error('errornoadmin', 'tool_sync');
}

$PAGE->requires->js('/admin/tool/sync/courses/js.js');

$selection = optional_param_array('selection', '', PARAM_TEXT);
if ($selection) {
    create_course_reinitialisation_file($selection);
}

$url = $CFG->wwwroot.'/admin/tool/sync/courses/resetcourses_creator.php';
$PAGE->navigation->add(get_string('synchronization', 'tool_sync'), $CFG->wwwroot.'/admin/tool/sync/index.php');
$PAGE->navigation->add(get_string('buildresetfile', 'tool_sync'));
$PAGE->set_context(null);
$PAGE->set_url($url);
$PAGE->set_title("$site->shortname");
$PAGE->set_heading($site->fullname);
echo $OUTPUT->header();

echo $OUTPUT->heading(get_string('resetfilebuilder', 'tool_sync'));

?>

<form name="form_reset" method="post" action="#" onSubmit="return select_all(this)">
<center>
<table class="generaltable" width="80%">
<tr class="r0" valign="top">
    <th class="header c0" align="left">
        <?php echo get_string('shortname'); ?>
    </th>
    <th class="header c1" align="left">
        <?php echo get_string('fullname'); ?>
    </th>
    <th class="header c2" align="left" colspan="5">
        <?php echo get_string('roles'); ?>
    </th>
</tr>
<?php

$COURSE_SORTS = array(
    0 => 'idnumber',
    1 => 'shortname',
    2 => 'id'
);

$sortorder = $COURSE_SORTS[0 + @$CFG->tool_sync_course_resetfileidentifier];

$courses = sync_get_all_courses($sortorder);
$class = 'r0';
$distinctcourses = array();
foreach ($courses as $c){
    $class = ($class == 'r0') ? 'r1' : 'r0' ;
    if (@$prevc->shortname != $c->shortname) {
        echo '</tr>';
        echo '<tr valign="top" class="'.$class.'">';
        echo '<td align="left" class="c0">'.$c->shortname .'</td><td align="left" class="c1"> '.$c->fullname .'</td>';
    } else {
        echo "<td>$c->rolename : $c->people</td>";
    }
    $distinctcourses[$c->shortname] = $c;
    $prevc = $c;
}
?>

</table>
<hr width="90%"/>
<table width="90%">
    <tr valign="top">
        <td>
        </td>
        <td align="center">
            <?php print_string('choosecoursetoreset', 'tool_sync') ?>
        </td>
        <td align="center">
            <?php print_string('selecteditems', 'tool_sync') ?>
        </td>
    </tr>
    <tr valign="top">
        <td align="center">
            <select style="height:200px" name="courselist" multiple OnDblClick="javascript:selectcourses(this.form.courselist,this.form.selection)" >
                <?php 
                foreach ($distinctcourses as $c) {
                    switch (0 + @$CFG->tool_sync_course_resetfileidentifier) {
                        case 0 : $cid = $c->idnumber; break;
                        case 1 : $cid = $c->shortname; break;
                        case 2 : $cid = $c->id; break;
                    }
                    echo '<option value="'.$cid.'">('.$c->idnumber.') '.$c->shortname.' - '.$c->fullname.'</option>';
                }
                ?>
            </select>
        </td>
        <td align="center">
            <table>
                <tr valign="top">
                    <td>
                        <input class="button" type="button" name="select" value=" >> " OnClick="javascript:selectcourses(this.form.courselist,this.form.selection)">
                    </td>
                </tr>
                <tr>
                    <td>
                        <input class="button" type="button" name="deselect" value=" << " OnClick="javascript:selectcourses(this.form.selection,this.form.courselist)">
                    </td>
                </tr>
            </table>
        </td>
        <td align="center">
            <select name="selection" multiple  style="height:200px" OnDblClick="javascript:selectcourses(this.form.selection,this.form.courselist)"></select>
        </td>
    </tr>
</table>
<p><input type="submit" value="<?php print_string('generate', 'tool_sync') ?>"/></p>
</center>
</form>

<?php

sync_print_return_button();

echo $OUTPUT->footer();
